var displayImgDimensionClassName = "twd-display-img-dimension";
var displayImgDimensionLocale = null;
var displayImgDimensionUpdating = null;
var displayImgDimensionTimeout = null;

// provides the current content window
function displayImgDimensionGetContentWindow() {
    return window;
}

// provides all the documents under a frame
function displayImgDimensionGetDocuments(frame) {
    var documents = [];
    // check if the frame is set
    if (frame) {
        var frames = frame.frames;
        // attempt to to access the frame document
        try {
            // check if the frame document exists
            if (frame.document) {
                documents.push(frame.document);
            }
        } catch (exception) {
            // Ignore
        }
        // go through the frames
        for (var i = 0, l = frames.length; i < l; i++) {
            documents = documents.concat(displayImgDimensionGetDocuments(frames[i]));
        }
    }
    return documents;
}

// Resize the dimensions for all images
function displayImgDimensionResizeImgDimension() {
    // check if there is a timeout set
    if (displayImgDimensionTimeout) {
        window.clearTimeout(displayImgDimensionTimeout);
        displayImgDimensionTimeout = null;
    }

    // check if the image dimensions are already updating
    if (displayImgDimensionUpdating) {
        displayImgDimensionTimeout = window.setTimeout(function() {
            displayImgDimensionResizeImgDimension();
        }, 0);
    } else {
        var documents = displayImgDimensionGetDocuments(displayImgDimensionGetContentWindow());
        // go through the documents
        for (var i = 0, l = documents.length; i < l; i++) {
            displayImgDimensionUpdateImgDimension(documents[i]);
        }
    }
}

// Updates the dimensions for all images
function displayImgDimensionUpdateImgDimension(contentDocument) {
    var image = null;
    var images = contentDocument.querySelectorAll("img");
    var spanElement = null;
    var text = null;
    displayImgDimensionUpdating = true;

    displayImgDimensionRemoveMatchingElement("." + displayImgDimensionClassName, contentDocument);

    // go through the images
    for (var i = 0, l = images.length; i < l; i++) {
        image = images[i];
        text = displayImgDimensionFormatDimension(image.width, image.height, displayImgDimensionLocale);
        // check if the text is set
        if (text) {
            spanElement = window.document.createElement("span");
            spanElement.setAttribute("class", displayImgDimensionClassName);
            spanElement.appendChild(window.document.createTextNode(text));
            image.parentNode.insertBefore(spanElement, image);
        }
    }
    displayImgDimensionUpdating = false;
}

// Removes all matching elements from a document
function displayImgDimensionRemoveMatchingElement(selector, contentDocument) {
    var matchingElement = null;
    var matchingElements = contentDocument.querySelectorAll(selector);
    // go through the matching elements
    for (var index1 = 0, len1 = matchingElements.length; index1 < len1; index1++) {
        matchingElement = matchingElements[index1];
        // check if the matching element has a parent node
        if (matchingElement.parentNode) {
            matchingElement.parentNode.removeChild(matchingElement);
        }
    }
}

// formats dimensions
function displayImgDimensionFormatDimension(width, height, dimensionText) {
    // check if the width and height are set
    if (width && height) {
        return dimensionText.width + " = " + width + "px " + dimensionText.height + " = " + height + "px";
    } else if (width) {
        return dimensionText.width + " = " + width + "px";
    } else if (height) {
        return dimensionText.height + " = " + height + "px";
    }
    return "";
}

//
//  displays the dimensions for all images
//
// eslint-disable-next-line no-unused-vars
function displayImgDimension() {
    // eslint-disable-next-line no-undef
    var docList = generateDocumentList();
    var dimensionText = {
        "width": "width",
        "height": "height",
    };
    var contentDocument = null;
    // go through doc list
    for (var index1 = 0, len1 = docList.length; index1 < len1; index1++) {
        contentDocument = docList[index1];
        // check if displaying the image dimensions
        displayImgDimensionLocale = dimensionText;
        window.addEventListener("resize", displayImgDimensionResizeImgDimension, false);
        displayImgDimensionUpdateImgDimension(contentDocument);
    }
}

//
//  removes the dimension of all images
//
// eslint-disable-next-line no-unused-vars
function displayImgDimensionUndo(display, documents, locale) {
    // eslint-disable-next-line no-undef
    var docList = generateDocumentList();
    var contentDocument = null;
    // go through doc list
    for (var index1 = 0, len1 = docList.length; index1 < len1; index1++) {
        contentDocument = docList[index1];
        // null if not displaying anf image dimension locale
        displayImgDimensionLocale = null;
        window.removeEventListener("resize", displayImgDimensionResizeImgDimension, false);
        displayImgDimensionRemoveMatchingElement("." + displayImgDimensionClassName, contentDocument);
    }
}