// content script for outlining elements with background imgs
var outlineElemWithBackgroundImgClassName = "twd-outline-background-img";

// provides property css value
// Gets the property CSS value for a computed style
function outlineElemWithBackgroundImgGetPropertyCSSValue(computedStyle, property) {
    var cssProperty = null;
    // check if the computed style is set
    if (computedStyle) {
        // attempt to to get the computed style (fails in newer versions of Chrome)
        try {
            cssProperty = computedStyle.getPropertyCSSValue(property);
        } catch (exception) {
            cssProperty = computedStyle.getPropertyValue(property);
        }
    }
    return cssProperty;
}

// provides the CSS text from a property
function outlineElemWithBackgroundImgGetCSSText(property) {
    // check if the property is set
    if (property) {
        // check if the property has CSS text
        if (property.cssText) {
            return property.cssText;
        }
        return property;
    }
    return null;
}

// provides true if an element has the specified class
function outlineElemWithBackgroundImgHasClass(element, className) {
    // check if the element and class name are set
    if (element && className) {
        var classes = element.className;
        // check if the classes are on an SVG
        if (classes instanceof SVGAnimatedString) {
            classes = classes.baseVal;
        }
        classes = classes.split(" ");
        // go through the classes
        for (var i = 0, l = classes.length; i < l; i++) {
            // check if the classes match
            if (className == classes[i]) {
                return true;
            }
        }
    }
    return false;
}

// add a class to an element
function outlineElemWithBackgroundImgAddClass(element, className) {
    // check if the element and class name are set and the element does not already have this class
    if (element && className && !outlineElemWithBackgroundImgHasClass(element, className)) {
        // check if the classes are on an SVG
        if (element.className instanceof SVGAnimatedString) {
            element.className.baseVal = (element.className.baseVal + " " + className).trim();
        } else {
            element.className = (element.className + " " + className).trim();
        }
    }
}

// provides a CSS primitive value
function outlineElemWithBackgroundImgGetCSSPrimitiveValue(type) {
    var cssPrimitiveValueExists = false;
    // attempt to to access the CSS primitive value
    try {
        // check if the CSS primitive value exists
        // eslint-disable-next-line no-undef
        if (CSSPrimitiveValue) {
            cssPrimitiveValueExists = true;
        }
    } catch (exception) {
        // Ignore
    }
    switch (type) {
        case "IDENT":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_IDENT : 21;
        case "NUMBER":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_NUMBER : 1;
        case "RGBCOLOR":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_RGBCOLOR : 25;
        case "URI":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_URI : 20;
        default:
            return null;
    }
}

// provides true if this CSS property is a URI
function outlineElemWithBackgroundImgIsCSSURI(property) {
    // check if the property is set
    if (property) {
        // check if the property has a primitive type
        if (property.primitiveType) {
            // check if the property primitive type is a URI
            if (property.primitiveType == outlineElemWithBackgroundImgGetCSSPrimitiveValue("URI")) {
                return true;
            }
        } else {
            var urlRegularExpression = /(?:\(['|"]?)(.*?)(?:['|"]?\))/;
            var uri = urlRegularExpression.exec(property);
            // check if the uri was found
            if (uri) {
                return true;
            }
        }
    }
    return false;
}

//
//  function that starts to outline elements with background images
//
// eslint-disable-next-line no-unused-vars
function outlineElemWithBackgroundImgFinal() {
    // eslint-disable-next-line no-undef
    var docList = generateDocumentList();
    var elementList = null;
    var element = null;
    var backImg = null;
    var contentDocument = null;
    for (var index2 = 0, len2 = docList.length; index2 < len2; index2++) {
        contentDocument = docList[index2];
        elementList = contentDocument.querySelectorAll("*");
        // go through the elements
        for (var index1 = 0, len1 = elementList.length; index1 < len1; index1++) {
            element = elementList[index1];
            backImg = outlineElemWithBackgroundImgGetCSSText(outlineElemWithBackgroundImgGetPropertyCSSValue(element.ownerDocument.defaultView.getComputedStyle(element, null), "background-image"));
            // check if this element has a background image and it is a URL
            if (backImg && outlineElemWithBackgroundImgIsCSSURI(backImg)) {
                outlineElemWithBackgroundImgAddClass(element, outlineElemWithBackgroundImgClassName);
            }
        }
    }
}

//
//  functions that removes outline from floating element
//
// eslint-disable-next-line no-unused-vars
function outlineElemWithBackgroundImgRemoveOutline() {
    // eslint-disable-next-line no-undef
    removeClassNameFromAllDocTypeElem(
        outlineElemWithBackgroundImgClassName
    );
}