// remove reload param from URL
function reloadImgRemoveReloadParameterFromURL(url) {
    if (url) {
        return url.replace(/(&|\?)twd-img-reload=\d+/, "");
    }
    return null;
}

// return CSS property
function reloadImgGetCSSProperty(property) {
    return property;
}

// Updates a reload image URL
function reloadImgUpdateReloadImageURL(imageURL) {
    if (!imageURL) {
        return;
    }
    var newImageURL = reloadImgRemoveReloadParameterFromURL(imageURL);
    // console.log("imageURL is");
    // console.log(imageURL);
    // console.log("newImageURL is");
    // console.log(newImageURL);
    // when img does not have query parameters
    if (newImageURL.indexOf("?") == -1) {
        newImageURL += "?";
    } else {
        newImageURL += "&";
    }
    return newImageURL + "twd-img-reload=" + new Date().getTime();
}

// get the CSS prop value for a computed style
function reloadImgGetPropertyCSSValue(computedStyle, property) {
    var cssProperty = null;
    // check if style is set
    if (computedStyle) {
        // get the computed style
        // fails in new versions of chrome
        try {
            cssProperty = computedStyle.getPropertyCSSValue(property);
        } catch (exception) {
            cssProperty = computedStyle.getPropertyValue(property);
        }
    }
    return cssProperty;
}

// return the CSS primitive val
function reloadImgGetCSSPrimitiveValue(type) {
    var cssPrimitiveValueExists = false;
    // attempt to to access the primitive value
    try {
        // check if the primitive val exist
        // eslint-disable-next-line no-undef
        if (CSSPrimitiveValue) {
            cssPrimitiveValueExists = true;
        }
    } catch (e) {
        // error while accessing the frame
    }
    switch (type) {
        case "IDENT":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_IDENT : 21;
        case "NUMBER":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_NUMBER : 1;
        case "RGBCOLOR":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_RGBCOLOR : 25;
        case "URI":
            // eslint-disable-next-line no-undef
            return cssPrimitiveValueExists ? CSSPrimitiveValue.CSS_URI : 20;
        default:
            return null;
    }
}

// return true if this CSS prop is a URI
function reloadImgIsCSSURI(property) {
    if (property) {
        // check if the prop has a primitive type
        if (property.primitiveType) {
            // property primitive type is a URI
            if (property.primitiveType == reloadImgGetCSSPrimitiveValue("URI")) {
                return true;
            }
        } else {
            var urlRegularExpression = /(?:\(['|"]?)(.*?)(?:['|"]?\))/;
            var uri = urlRegularExpression.exec(property);
            // check if URI is found
            if (uri) {
                return true;
            }
        }
    }
    return false;
}

// ret the CSS URI from a property
function reloadImgGetCSSURI(property) {
    // check if the property is set
    if (property) {
        // check if the property has a primitive type
        if (property.primitiveType) {
            return property.getStringValue();
        } else { // eslint-disable-line no-else-return
            var urlRegularExpression = /(?:\(['|"]?)(.*?)(?:['|"]?\))/;
            var uri = urlRegularExpression.exec(property);
            // check if the uri was found
            if (uri) {
                return uri[1];
            }
        }
    }
    return null;
}

function reloadImgGenerateDocumentList() {
    // add normal documents
    var documentList = [];
    documentList.push(window.document);
    // add all iframe documents
    var iframeList = document.querySelectorAll("iframe");
    for (var index = 0, len = iframeList.length; index < len; index++) {
        try {
            if (iframeList[index].contentWindow.document) {
                // check if set then add to the list
                documentList.push(iframeList[index].contentWindow.document);
            }
        } catch (e) {
            // for cross origin error
        }
    }
    // add shadow roots to the document list
    var allNodes = document.querySelectorAll("*");
    for (var index1 = 0, len1 = allNodes.length; index1 < len1; index1++) {
        if (allNodes[index1].shadowRoot) {
            documentList.push(allNodes[index1].shadowRoot);
        }
    }
    return documentList;
}

function reloadImgAll() {
    var documentList = reloadImgGenerateDocumentList();
    var computedStyle = null;
    var contentDocument = null;
    var imgURL = null;
    var node = null;
    var styleImage = null;
    var nodeList = null;

    for (var index2 = 0, len2 = documentList.length; index2 < len2; index2++) {
        contentDocument = documentList[index2];
        nodeList = contentDocument.querySelectorAll("*");
        for (var index3 = 0, len3 = nodeList.length; index3 < len3; index3++) {
            node = nodeList[index3];
            // check if this is an img element
            if (node.tagName.toLowerCase() == "img" || node.tagName.toLowerCase() == "input" && node.src && node.type && node.type.toLowerCase() == "image") {
                imgURL = node.src;
                // data URI check
                if (imgURL.indexOf("data:") === 0) {
                    // check if this is a data URI then continue
                    continue;
                }
                // check if this is not a chrome img
                if (imgURL.indexOf("chrome://") !== 0) {
                    node.src = reloadImgUpdateReloadImageURL(imgURL);
                }
            } else if (node.tagName.toLowerCase() == "link" && node.href && node.href.indexOf("chrome://") !== 0 && node.rel && node.rel.indexOf("icon") != -1) {
                node.href = reloadImgUpdateReloadImageURL(node.href);
            } else {
                computedStyle = node.ownerDocument.defaultView.getComputedStyle(node, null);
                // check if the computed style is set
                if (computedStyle) {
                    styleImage = reloadImgGetCSSProperty(reloadImgGetPropertyCSSValue(computedStyle, "background-image"));
                    // check if this element has a background img and it is a URI
                    if (reloadImgIsCSSURI(styleImage)) {
                        imgURL = reloadImgGetCSSURI(styleImage);
                        // check if this is not a chrome image
                        if (imgURL.indexOf("chrome://") !== 0) {
                            node.style.backgroundImage = reloadImgUpdateReloadImageURL(imgURL);
                        }
                    }
                    styleImage = reloadImgGetPropertyCSSValue(computedStyle, "list-style-image");
                    // check if this element has a list style img and it is a URI
                    if (reloadImgIsCSSURI(styleImage)) {
                        imgURL = reloadImgGetCSSURI(styleImage);
                        // check if this is not a chrome image
                        if (imgURL.indexOf("chrome://") !== 0) {
                            node.style.listStyleImage = reloadImgUpdateReloadImageURL(imgURL);
                        }
                    }
                }
            }
        }
    }
}

// reload image in a document
// eslint-disable-next-line no-unused-vars
function reloadImgFinal() {
    reloadImgAll();
}